import unittest

from .cn2an import Cn2An


class Cn2anTest(unittest.TestCase):
    def setUp(self) -> None:
        self.strict_data_dict = {
            "零": 0,
            "一": 1,
            "十": 10,
            "十一": 11,
            "一十一": 11,
            "二十": 20,
            "二十一": 21,
            "一百": 100,
            "一百零一": 101,
            "一百一十": 110,
            "一百一十一": 111,
            "一千": 1000,
            "一千一百": 1100,
            "一千一百一十": 1110,
            "一千一百一十一": 1111,
            "一千零一十": 1010,
            "一千零十": 1010,
            "一千零十一": 1011,
            "一千零一十一": 1011,
            "一千零一": 1001,
            "一千一百零一": 1101,
            "一万一千一百一十一": 11111,
            "一十一万一千一百一十一": 111111,
            "一百一十一万一千一百一十一": 1111111,
            "一千一百一十一万一千一百一十一": 11111111,
            "一亿一千一百一十一万一千一百一十一": 111111111,
            "一十一亿一千一百一十一万一千一百一十一": 1111111111,
            "一百一十一亿一千一百一十一万一千一百一十一": 11111111111,
            "一千一百一十一亿一千一百一十一万一千一百一十一": 111111111111,
            "一千一百一十一万一千一百一十一亿一千一百一十一万一千一百一十一": 1111111111111111,
            "壹": 1,
            "拾": 10,
            "拾壹": 11,
            "壹拾壹": 11,
            "壹佰壹拾壹": 111,
            "壹仟壹佰壹拾壹": 1111,
            "壹万壹仟壹佰壹拾壹": 11111,
            "壹拾壹万壹仟壹佰壹拾壹": 111111,
            "壹佰壹拾壹万壹仟壹佰壹拾壹": 1111111,
            "壹仟壹佰壹拾壹万壹仟壹佰壹拾壹": 11111111,
            "壹亿壹仟壹佰壹拾壹万壹仟壹佰壹拾壹": 111111111,
            "壹拾壹亿壹仟壹佰壹拾壹万壹仟壹佰壹拾壹": 1111111111,
            "壹佰壹拾壹亿壹仟壹佰壹拾壹万壹仟壹佰壹拾壹": 11111111111,
            "壹仟壹佰壹拾壹亿壹仟壹佰壹拾壹万壹仟壹佰壹拾壹": 111111111111,
            "壹拾壹元整": 11,
            "壹佰壹拾壹圆整": 111,
            "壹拾壹元正": 11,
            "壹拾壹圆正": 11,
            "壹拾壹元壹角": 11.1,
            "壹拾壹元壹角壹分": 11.11,
            "十万": 100000,
            "十万零一": 100001,
            "一万零一": 10001,
            "一万零一十一": 10011,
            "一万零一百一十一": 10111,
            "一万零百一十一": 10111,
            "一十万零一": 100001,
            "一百万零一": 1000001,
            "一千万零一": 10000001,
            "一千零一万一千零一": 10011001,
            "一千零一万零一": 10010001,
            "一亿零一": 100000001,
            "一十亿零一": 1000000001,
            "一百亿零一": 10000000001,
            "一千零一亿一千零一万一千零一": 100110011001,
            "一千亿一千万一千零一": 100010001001,
            "一千亿零一": 100000000001,
            "零点零零零零零零零零零零零零零零一": 0.000000000000001,
            "零点零零零零零零零零零零零零零一": 0.00000000000001,
            "零点零零零零零零零零零零零零一": 0.0000000000001,
            "零点零零零零零零零零零零零一": 0.000000000001,
            "零点零零零零零零零零零零一": 0.00000000001,
            "零点零零零零零零零零零一": 0.0000000001,
            "零点零零零零零零零零一": 0.000000001,
            "零点零零零零零零零一": 0.00000001,
            "零点零零零零零零一": 0.0000001,
            "零点零零零零零一": 0.000001,
            "零点零零零零一": 0.00001,
            "零点零零零一": 0.0001,
            "零点零零一": 0.001,
            "零点零一": 0.01,
            "零点一": 0.1,
            "负一": -1,
            "负二": -2,
            "负十": -10,
            "负十一": -11,
            "负一十一": -11,
            # 古语
            "廿二": 22,
        }

        self.normal_data_dict = {
            "一一": 11,
            "一一一": 111,
            "壹壹": 11,
            "壹壹壹": 111,
            "零点零": 0,
            "零点零零": 0,
            "一七二零": 1720,
            "一七二零点一": 1720.1,
            "一七二零点一三四": 1720.134,
            "一二三": 123,
            "负零点一零": -0.1,
            "负一七二零": -1720,
            "负一七二零点一": -1720.1,
            # 口语
            "三万五": 35000,
            "十三万五": 135000,
            "两千六": 2600,
            "一百二": 120,
            "一百二十万三": 1203000,
            # 繁体
            "兩千六": 2600,
            # 大写
            "壹拾壹元": 11,
            "壹佰壹拾壹圆": 111,
            "壹拾壹圆": 11,
            # 特殊
            "〇": 0,
        }

        self.smart_data_dict = {
            "100万": 1000000,
            "100万三千": 1003000,
            "200亿零四千230": 20000004230,
            "一百点123": 100.123,
            "10.1万": 101000,
            "-10.1万": -101000,
            "35.1亿": 3510000000,
            "10.1": 10.1,
            "-10.1": -10.1,
        }

        self.error_smart_datas = [
            "10.1万零100",
            "10..1万",
        ]

        self.error_normal_datas = [
            "零点",
            "点零",
            "零点点",
            "零点零大",
        ]
        self.error_normal_datas.extend(self.error_smart_datas)
        self.error_normal_datas.extend(list(self.smart_data_dict.keys()))

        self.error_strict_datas = [
            "一一",
            "壹壹",
            "零点",
            "点零",
            "点一",
            "百十一",
            "十一十二",
            "负十一十二",
            "十七十八",
        ]
        self.error_strict_datas.extend(self.error_normal_datas)
        self.error_strict_datas.extend(list(self.normal_data_dict.keys()))

        # 不可修改位置
        self.normal_data_dict.update(self.strict_data_dict)
        self.smart_data_dict.update(self.normal_data_dict)

        self.ca = Cn2An()

    def test_cn2an(self) -> None:
        for strict_item in self.strict_data_dict.keys():
            self.assertEqual(self.ca.cn2an(strict_item, "strict"),
                             self.strict_data_dict[strict_item])

        for normal_item in self.normal_data_dict.keys():
            self.assertEqual(self.ca.cn2an(normal_item, "normal"),
                             self.normal_data_dict[normal_item])

        for smart_item in self.smart_data_dict.keys():
            self.assertEqual(self.ca.cn2an(smart_item, "smart"),
                             self.smart_data_dict[smart_item])

        for error_strict_item in self.error_strict_datas:
            try:
                self.ca.cn2an(error_strict_item)
            except ValueError as e:
                self.assertEqual(type(e), ValueError)
            else:
                raise Exception(f'ValueError not raised: {error_strict_item}')

        for error_normal_item in self.error_normal_datas:
            try:
                self.ca.cn2an(error_normal_item)
            except ValueError as e:
                self.assertEqual(type(e), ValueError)
            else:
                raise Exception(f'ValueError not raised: {error_normal_item}')

        for error_smart_item in self.error_smart_datas:
            try:
                self.ca.cn2an(error_smart_item)
            except ValueError as e:
                self.assertEqual(type(e), ValueError)
            else:
                raise Exception(f'ValueError not raised: {error_smart_item}')


if __name__ == '__main__':
    unittest.main()



from time import time

numeral_list = {0: "零",
                1: "一",
                2: "二",
                3: "三",
                4: "四",
                5: "五",
                6: "六",
                7: "七",
                8: "八",
                9: "九"}
unit_list = ["", "十", "百", "千", "万", "十", "百", "千", "亿", "十", "百", "千", "万", "十", "百", "千"]


def an2cn(integer_data: int) -> str:
    integer_data = str(integer_data)
    len_integer_data = len(integer_data)
    output_an = ""
    for i, d in enumerate(integer_data):
        if int(d):
            output_an += numeral_list[int(d)] + unit_list[len_integer_data - i - 1]
        else:
            if not (len_integer_data - i - 1) % 4:
                output_an += numeral_list[int(d)] + unit_list[len_integer_data - i - 1]

            if i > 0 and not output_an[-1] == "零":
                output_an += numeral_list[int(d)]

    output_an = output_an.replace("零零", "零").replace("零万", "万").replace("零亿", "亿").strip("零")
    return output_an


def run_an2cn_core_ten_thousand_times():
    t_start = time()
    for _ in range(1000000):
        assert an2cn(9876543298765432) == "九千八百七十六万五千四百三十二亿九千八百七十六万五千四百三十二"
    t_end = time()
    print(round(t_end - t_start, 3))


if __name__ == '__main__':
    run_an2cn_core_ten_thousand_times()



from typing import Union
from warnings import warn

from proces import preprocess

from .conf import NUMBER_LOW_AN2CN, NUMBER_UP_AN2CN, UNIT_LOW_ORDER_AN2CN, UNIT_UP_ORDER_AN2CN


class An2Cn(object):
    def __init__(self) -> None:
        self.all_num = "0123456789"
        self.number_low = NUMBER_LOW_AN2CN
        self.number_up = NUMBER_UP_AN2CN
        self.mode_list = ["low", "up", "rmb", "direct"]

    def an2cn(self, inputs: Union[str, int, float] = None, mode: str = "low") -> str:
        """阿拉伯数字转中文数字

        :param inputs: 阿拉伯数字
        :param mode: low 小写数字，up 大写数字，rmb 人民币大写，direct 直接转化
        :return: 中文数字
        """
        if inputs is not None and inputs != "":
            if mode not in self.mode_list:
                raise ValueError(f"mode 仅支持 {str(self.mode_list)} ！")

            # 将数字转化为字符串，这里会有Python会自动做转化
            # 1. -> 1.0 1.00 -> 1.0 -0 -> 0
            if not isinstance(inputs, str):
                inputs = self.__number_to_string(inputs)

            # 数据预处理：
            # 1. 繁体转简体
            # 2. 全角转半角
            inputs = preprocess(inputs, pipelines=[
                "traditional_to_simplified",
                "full_angle_to_half_angle"
            ])

            # 检查数据是否有效
            self.__check_inputs_is_valid(inputs)

            # 判断正负
            if inputs[0] == "-":
                sign = "负"
                inputs = inputs[1:]
            else:
                sign = ""

            if mode == "direct":
                output = self.__direct_convert(inputs)
            else:
                # 切割整数部分和小数部分
                split_result = inputs.split(".")
                len_split_result = len(split_result)
                if len_split_result == 1:
                    # 不包含小数的输入
                    integer_data = split_result[0]
                    if mode == "rmb":
                        output = self.__integer_convert(integer_data, "up") + "元整"
                    else:
                        output = self.__integer_convert(integer_data, mode)
                elif len_split_result == 2:
                    # 包含小数的输入
                    integer_data, decimal_data = split_result
                    if mode == "rmb":
                        int_data = self.__integer_convert(integer_data, "up")
                        dec_data = self.__decimal_convert(decimal_data, "up")
                        len_dec_data = len(dec_data)

                        if len_dec_data == 0:
                            output = int_data + "元整"
                        elif len_dec_data == 1:
                            raise ValueError(f"异常输出：{dec_data}")
                        elif len_dec_data == 2:
                            if dec_data[1] != "零":
                                if int_data == "零":
                                    output = dec_data[1] + "角"
                                else:
                                    output = int_data + "元" + dec_data[1] + "角"
                            else:
                                output = int_data + "元整"
                        else:
                            if dec_data[1] != "零":
                                if dec_data[2] != "零":
                                    if int_data == "零":
                                        output = dec_data[1] + "角" + dec_data[2] + "分"
                                    else:
                                        output = int_data + "元" + dec_data[1] + "角" + dec_data[2] + "分"
                                else:
                                    if int_data == "零":
                                        output = dec_data[1] + "角"
                                    else:
                                        output = int_data + "元" + dec_data[1] + "角"
                            else:
                                if dec_data[2] != "零":
                                    if int_data == "零":
                                        output = dec_data[2] + "分"
                                    else:
                                        output = int_data + "元" + "零" + dec_data[2] + "分"
                                else:
                                    output = int_data + "元整"
                    else:
                        output = self.__integer_convert(integer_data, mode) + self.__decimal_convert(decimal_data, mode)
                else:
                    raise ValueError(f"输入格式错误：{inputs}！")
        else:
            raise ValueError("输入数据为空！")

        return sign + output

    def __direct_convert(self, inputs: str) -> str:
        _output = ""
        for d in inputs:
            if d == ".":
                _output += "点"
            else:
                _output += self.number_low[int(d)]
        return _output

    @staticmethod
    def __number_to_string(number_data: Union[int, float]) -> str:
        # 小数处理：python 会自动把 0.00005 转化成 5e-05，因此 str(0.00005) != "0.00005"
        string_data = str(number_data)
        if "e" in string_data:
            string_data_list = string_data.split("e")
            string_key = string_data_list[0]
            string_value = string_data_list[1]
            if string_value[0] == "-":
                string_data = "0." + "0" * (int(string_value[1:]) - 1) + string_key
            else:
                string_data = string_key + "0" * int(string_value)
        return string_data

    def __check_inputs_is_valid(self, check_data: str) -> None:
        # 检查输入数据是否在规定的字典中
        all_check_keys = self.all_num + ".-"
        for data in check_data:
            if data not in all_check_keys:
                raise ValueError(f"输入的数据不在转化范围内：{data}！")

    def __integer_convert(self, integer_data: str, mode: str) -> str:
        if mode == "low":
            numeral_list = NUMBER_LOW_AN2CN
            unit_list = UNIT_LOW_ORDER_AN2CN
        elif mode == "up":
            numeral_list = NUMBER_UP_AN2CN
            unit_list = UNIT_UP_ORDER_AN2CN
        else:
            raise ValueError(f"error mode: {mode}")

        # 去除前面的 0，比如 007 => 7
        integer_data = str(int(integer_data))

        len_integer_data = len(integer_data)
        if len_integer_data > len(unit_list):
            raise ValueError(f"超出数据范围，最长支持 {len(unit_list)} 位")

        output_an = ""
        for i, d in enumerate(integer_data):
            if int(d):
                output_an += numeral_list[int(d)] + unit_list[len_integer_data - i - 1]
            else:
                if not (len_integer_data - i - 1) % 4:
                    output_an += numeral_list[int(d)] + unit_list[len_integer_data - i - 1]

                if i > 0 and not output_an[-1] == "零":
                    output_an += numeral_list[int(d)]

        output_an = output_an.replace("零零", "零").replace("零万", "万").replace("零亿", "亿").replace("亿万", "亿") \
            .strip("零")

        # 解决「一十几」问题
        if output_an[:2] in ["一十"]:
            output_an = output_an[1:]

        # 0 - 1 之间的小数
        if not output_an:
            output_an = "零"

        return output_an

    def __decimal_convert(self, decimal_data: str, o_mode: str) -> str:
        len_decimal_data = len(decimal_data)

        if len_decimal_data > 16:
            warn(f"注意：小数部分长度为 {len_decimal_data} ，将自动截取前 16 位有效精度！")
            decimal_data = decimal_data[:16]

        if len_decimal_data:
            output_an = "点"
        else:
            output_an = ""

        if o_mode == "low":
            numeral_list = NUMBER_LOW_AN2CN
        elif o_mode == "up":
            numeral_list = NUMBER_UP_AN2CN
        else:
            raise ValueError(f"error mode: {o_mode}")

        for data in decimal_data:
            output_an += numeral_list[int(data)]
        return output_an



import re
from warnings import warn
from typing import Union

from proces import preprocess

from .an2cn import An2Cn
from .conf import NUMBER_CN2AN, UNIT_CN2AN, STRICT_CN_NUMBER, NORMAL_CN_NUMBER, NUMBER_LOW_AN2CN, UNIT_LOW_AN2CN


class Cn2An(object):
    def __init__(self) -> None:
        self.all_num = "".join(list(NUMBER_CN2AN.keys()))
        self.all_unit = "".join(list(UNIT_CN2AN.keys()))
        self.strict_cn_number = STRICT_CN_NUMBER
        self.normal_cn_number = NORMAL_CN_NUMBER
        self.check_key_dict = {
            "strict": "".join(self.strict_cn_number.values()) + "点负",
            "normal": "".join(self.normal_cn_number.values()) + "点负",
            "smart": "".join(self.normal_cn_number.values()) + "点负" + "01234567890.-"
        }
        self.pattern_dict = self.__get_pattern()
        self.ac = An2Cn()
        self.mode_list = ["strict", "normal", "smart"]
        self.yjf_pattern = re.compile(fr"^.*?[元圆][{self.all_num}]角([{self.all_num}]分)?$")
        self.pattern1 = re.compile(fr"^-?\d+(\.\d+)?[{self.all_unit}]?$")
        self.ptn_all_num = re.compile(f"^[{self.all_num}]+$")
        # "十?" is for special case "十一万三"
        self.ptn_speaking_mode = re.compile(f"^([{self.all_num}]{{0,2}}[{self.all_unit}])+[{self.all_num}]$")

    def cn2an(self, inputs: Union[str, int, float] = None, mode: str = "strict") -> Union[float, int]:
        """中文数字转阿拉伯数字

        :param inputs: 中文数字、阿拉伯数字、中文数字和阿拉伯数字
        :param mode: strict 严格，normal 正常，smart 智能
        :return: 阿拉伯数字
        """
        if inputs is not None or inputs == "":
            if mode not in self.mode_list:
                raise ValueError(f"mode 仅支持 {str(self.mode_list)} ！")

            # 将数字转化为字符串
            if not isinstance(inputs, str):
                inputs = str(inputs)

            # 数据预处理：
            # 1. 繁体转简体
            # 2. 全角转半角
            inputs = preprocess(inputs, pipelines=[
                "traditional_to_simplified",
                "full_angle_to_half_angle"
            ])

            # 特殊转化 廿
            inputs = inputs.replace("廿", "二十")

            # 检查输入数据是否有效
            sign, integer_data, decimal_data, is_all_num = self.__check_input_data_is_valid(inputs, mode)

            # smart 下的特殊情况
            if sign == 0:
                return integer_data
            else:
                if not is_all_num:
                    if decimal_data is None:
                        output = self.__integer_convert(integer_data)
                    else:
                        output = self.__integer_convert(integer_data) + self.__decimal_convert(decimal_data)
                        # fix 1 + 0.57 = 1.5699999999999998
                        output = round(output, len(decimal_data))
                else:
                    if decimal_data is None:
                        output = self.__direct_convert(integer_data)
                    else:
                        output = self.__direct_convert(integer_data) + self.__decimal_convert(decimal_data)
                        # fix 1 + 0.57 = 1.5699999999999998
                        output = round(output, len(decimal_data))
        else:
            raise ValueError("输入数据为空！")

        return sign * output

    def __get_pattern(self) -> dict:
        # 整数严格检查
        _0 = "[零]"
        _1_9 = "[一二三四五六七八九]"
        _10_99 = f"{_1_9}?[十]{_1_9}?"
        _1_99 = f"({_10_99}|{_1_9})"
        _100_999 = f"({_1_9}[百]([零]{_1_9})?|{_1_9}[百]{_10_99})"
        _1_999 = f"({_100_999}|{_1_99})"
        _1000_9999 = f"({_1_9}[千]([零]{_1_99})?|{_1_9}[千]{_100_999})"
        _1_9999 = f"({_1000_9999}|{_1_999})"
        _10000_99999999 = f"({_1_9999}[万]([零]{_1_999})?|{_1_9999}[万]{_1000_9999})"
        _1_99999999 = f"({_10000_99999999}|{_1_9999})"
        _100000000_9999999999999999 = f"({_1_99999999}[亿]([零]{_1_99999999})?|{_1_99999999}[亿]{_10000_99999999})"
        _1_9999999999999999 = f"({_100000000_9999999999999999}|{_1_99999999})"
        str_int_pattern = f"^({_0}|{_1_9999999999999999})$"
        nor_int_pattern = f"^({_0}|{_1_9999999999999999})$"

        str_dec_pattern = "^[零一二三四五六七八九]{0,15}[一二三四五六七八九]$"
        nor_dec_pattern = "^[零一二三四五六七八九]{0,16}$"

        for str_num in self.strict_cn_number.keys():
            str_int_pattern = str_int_pattern.replace(str_num, self.strict_cn_number[str_num])
            str_dec_pattern = str_dec_pattern.replace(str_num, self.strict_cn_number[str_num])
        for nor_num in self.normal_cn_number.keys():
            nor_int_pattern = nor_int_pattern.replace(nor_num, self.normal_cn_number[nor_num])
            nor_dec_pattern = nor_dec_pattern.replace(nor_num, self.normal_cn_number[nor_num])

        pattern_dict = {
            "strict": {
                "int": re.compile(str_int_pattern),
                "dec": re.compile(str_dec_pattern)
            },
            "normal": {
                "int": re.compile(nor_int_pattern),
                "dec": re.compile(nor_dec_pattern)
            }
        }
        return pattern_dict

    def __copy_num(self, num):
        cn_num = ""
        for n in num:
            cn_num += NUMBER_LOW_AN2CN[int(n)]
        return cn_num

    def __check_input_data_is_valid(self, check_data: str, mode: str) -> (int, str, str, bool):
        # 去除 元整、圆整、元正、圆正
        stop_words = ["元整", "圆整", "元正", "圆正"]
        for word in stop_words:
            if check_data[-2:] == word:
                check_data = check_data[:-2]

        # 去除 元、圆
        if mode != "strict":
            normal_stop_words = ["圆", "元"]
            for word in normal_stop_words:
                if check_data[-1] == word:
                    check_data = check_data[:-1]

        # 处理元角分
        result = self.yjf_pattern.search(check_data)
        if result:
            check_data = check_data.replace("元", "点").replace("角", "").replace("分", "")

        # 处理特殊问法：一千零十一 一万零百一十一
        if "零十" in check_data:
            check_data = check_data.replace("零十", "零一十")
        if "零百" in check_data:
            check_data = check_data.replace("零百", "零一百")

        for data in check_data:
            if data not in self.check_key_dict[mode]:
                raise ValueError(f"当前为{mode}模式，输入的数据不在转化范围内：{data}！")

        # 确定正负号
        if check_data[0] == "负":
            check_data = check_data[1:]
            sign = -1
        else:
            sign = 1

        if "点" in check_data:
            split_data = check_data.split("点")
            if len(split_data) == 2:
                integer_data, decimal_data = split_data
                # 将 smart 模式中的阿拉伯数字转化成中文数字
                if mode == "smart":
                    integer_data = re.sub(r"\d+", lambda x: self.ac.an2cn(x.group()), integer_data)
                    decimal_data = re.sub(r"\d+", lambda x: self.__copy_num(x.group()), decimal_data)
                    mode = "normal"
            else:
                raise ValueError("数据中包含不止一个点！")
        else:
            integer_data = check_data
            decimal_data = None
            # 将 smart 模式中的阿拉伯数字转化成中文数字
            if mode == "smart":
                # 10.1万 10.1
                result1 = self.pattern1.search(integer_data)
                if result1:
                    if result1.group() == integer_data:
                        if integer_data[-1] in UNIT_CN2AN.keys():
                            output = int(float(integer_data[:-1]) * UNIT_CN2AN[integer_data[-1]])
                        else:
                            output = float(integer_data)
                        return 0, output, None, None

                integer_data = re.sub(r"\d+", lambda x: self.ac.an2cn(x.group()), integer_data)
                mode = "normal"

        result_int = self.pattern_dict[mode]["int"].search(integer_data)
        if result_int:
            if result_int.group() == integer_data:
                if decimal_data is not None:
                    result_dec = self.pattern_dict[mode]["dec"].search(decimal_data)
                    if result_dec:
                        if result_dec.group() == decimal_data:
                            return sign, integer_data, decimal_data, False
                else:
                    return sign, integer_data, decimal_data, False
        else:
            if mode == "strict":
                raise ValueError(f"不符合格式的数据：{integer_data}")
            elif mode == "normal":
                # 纯数模式：一二三
                result_all_num = self.ptn_all_num.search(integer_data)
                if result_all_num:
                    if result_all_num.group() == integer_data:
                        if decimal_data is not None:
                            result_dec = self.pattern_dict[mode]["dec"].search(decimal_data)
                            if result_dec:
                                if result_dec.group() == decimal_data:
                                    return sign, integer_data, decimal_data, True
                        else:
                            return sign, integer_data, decimal_data, True

                # 口语模式：一万二，两千三，三百四，十三万六，一百二十五万三
                result_speaking_mode = self.ptn_speaking_mode.search(integer_data)
                if len(integer_data) >= 3 and result_speaking_mode and result_speaking_mode.group() == integer_data:
                    # len(integer_data)>=3: because the minimum length of integer_data that can be matched is 3
                    # to find the last unit
                    last_unit = result_speaking_mode.groups()[-1][-1]
                    _unit = UNIT_LOW_AN2CN[UNIT_CN2AN[last_unit] // 10]
                    integer_data = integer_data + _unit
                    if decimal_data is not None:
                        result_dec = self.pattern_dict[mode]["dec"].search(decimal_data)
                        if result_dec:
                            if result_dec.group() == decimal_data:
                                return sign, integer_data, decimal_data, False
                    else:
                        return sign, integer_data, decimal_data, False

        raise ValueError(f"不符合格式的数据：{check_data}")

    def __integer_convert(self, integer_data: str) -> int:
        # 核心
        output_integer = 0
        unit = 1
        ten_thousand_unit = 1
        for index, cn_num in enumerate(reversed(integer_data)):
            # 数值
            if cn_num in NUMBER_CN2AN:
                num = NUMBER_CN2AN[cn_num]
                output_integer += num * unit
            # 单位
            elif cn_num in UNIT_CN2AN:
                unit = UNIT_CN2AN[cn_num]
                # 判断出万、亿、万亿
                if unit % 10000 == 0:
                    # 万 亿
                    if unit > ten_thousand_unit:
                        ten_thousand_unit = unit
                    # 万亿
                    else:
                        ten_thousand_unit = unit * ten_thousand_unit
                        unit = ten_thousand_unit

                if unit < ten_thousand_unit:
                    unit = unit * ten_thousand_unit

                if index == len(integer_data) - 1:
                    output_integer += unit
            else:
                raise ValueError(f"{cn_num} 不在转化范围内")

        return int(output_integer)

    def __decimal_convert(self, decimal_data: str) -> float:
        len_decimal_data = len(decimal_data)

        if len_decimal_data > 16:
            warn(f"注意：小数部分长度为 {len_decimal_data} ，将自动截取前 16 位有效精度！")
            decimal_data = decimal_data[:16]
            len_decimal_data = 16

        output_decimal = 0
        for index in range(len(decimal_data) - 1, -1, -1):
            unit_key = NUMBER_CN2AN[decimal_data[index]]
            output_decimal += unit_key * 10 ** -(index + 1)

        # 处理精度溢出问题
        output_decimal = round(output_decimal, len_decimal_data)

        return output_decimal

    def __direct_convert(self, data: str) -> int:
        output_data = 0
        for index in range(len(data) - 1, -1, -1):
            unit_key = NUMBER_CN2AN[data[index]]
            output_data += unit_key * 10 ** (len(data) - index - 1)

        return output_data



__version__ = "0.5.21"

from .cn2an import Cn2An
from .an2cn import An2Cn
from .transform import Transform

cn2an = Cn2An().cn2an
an2cn = An2Cn().an2cn
transform = Transform().transform

__all__ = [
    "__version__",
    "cn2an",
    "an2cn",
    "transform"
]



# 数字映射
number_map = {
    "零": 0,
    "一": 1,
    "二": 2,
    "三": 3,
    "四": 4,
    "五": 5,
    "六": 6,
    "七": 7,
    "八": 8,
    "九": 9
}

# 单位映射
unit_map = {
    "十": 10,
    "百": 100,
    "千": 1000,
    "万": 10000,
    "亿": 100000000
}


# 正向遍历 1
def forward_cn2an_one(inputs):
    output = 0
    unit = 1
    num = 0
    for index, cn_num in enumerate(inputs):
        if cn_num in number_map:
            # 数字
            num = number_map[cn_num]
            # 最后的个位数字
            if index == len(inputs) - 1:
                output = output + num
        elif cn_num in unit_map:
            # 单位
            unit = unit_map[cn_num]
            # 累加
            output = output + num * unit
            num = 0
        else:
            raise ValueError(f"{cn_num} 不在转化范围内")

    return output


# 正向遍历 2
def forward_cn2an_two(inputs):
    output = 0
    unit = 1
    num = 0
    for index, cn_num in enumerate(inputs):
        if cn_num in number_map:
            # 数字
            num = number_map[cn_num]
            # 最后的个位数字
            if index == len(inputs) - 1:
                output = output + num
        elif cn_num in unit_map:
            # 单位
            unit = unit_map[cn_num]
            # 判断出万、亿，先把前面的累加再乘以单位万、亿
            if unit % 10000 == 0:
                output = (output + num) * unit
            else:
                # 累加
                output = output + num * unit
            num = 0
        else:
            raise ValueError(f"{cn_num} 不在转化范围内")

    return output


# 正向遍历 3
def forward_cn2an_three(inputs):
    output = 0
    unit = 1
    num = 0
    # 亿位以上的输出
    hundred_million_output = 0
    for index, cn_num in enumerate(inputs):
        if cn_num in number_map:
            # 数字
            num = number_map[cn_num]
            # 最后的个位数字
            if index == len(inputs) - 1:
                # 把亿位和中间输出以及个位上的一起加起来
                output = hundred_million_output + output + num
        elif cn_num in unit_map:
            # 单位
            unit = unit_map[cn_num]
            # 判断出万，前面的累加再乘以单位万
            if unit == 10000:
                output = (output + num) * unit
            # 判断出亿，前面累加乘以亿后赋值给 hundred_million_output， output 重置为 0
            elif unit == 100000000:
                hundred_million_output = (output + num) * unit
                output = 0
            else:
                # 累加
                output = output + num * unit
            num = 0
        else:
            raise ValueError(f"{cn_num} 不在转化范围内")

    return output


# 反向遍历 1
def backward_cn2an_one(inputs):
    output = 0
    unit = 1
    num = 0
    for index, cn_num in enumerate(reversed(inputs)):
        if cn_num in number_map:
            # 数字
            num = number_map[cn_num]
            # 累加
            output = output + num * unit
        elif cn_num in unit_map:
            # 单位
            unit = unit_map[cn_num]
        else:
            raise ValueError(f"{cn_num} 不在转化范围内")

    return output


# 反向遍历 2
def backward_cn2an_two(inputs):
    output = 0
    unit = 1
    # 万、亿的单位
    ten_thousand_unit = 1
    num = 0
    for index, cn_num in enumerate(reversed(inputs)):
        if cn_num in number_map:
            # 数字
            num = number_map[cn_num]
            # 累加
            output = output + num * unit
        elif cn_num in unit_map:
            # 单位
            unit = unit_map[cn_num]
            # 判断出万、亿
            if unit % 10000 == 0:
                ten_thousand_unit = unit

            if unit < ten_thousand_unit:
                unit = ten_thousand_unit * unit
        else:
            raise ValueError(f"{cn_num} 不在转化范围内")

    return output


# 反向遍历 3
def backward_cn2an_three(inputs):
    output = 0
    unit = 1
    # 万、亿的单位
    ten_thousand_unit = 1
    num = 0
    for index, cn_num in enumerate(reversed(inputs)):
        if cn_num in number_map:
            # 数字
            num = number_map[cn_num]
            # 累加
            output = output + num * unit
        elif cn_num in unit_map:
            # 单位
            unit = unit_map[cn_num]
            # 判断出万、亿
            if unit % 10000 == 0:
                # 万、亿
                if unit > ten_thousand_unit:
                    ten_thousand_unit = unit
                # 万亿
                else:
                    ten_thousand_unit = unit * ten_thousand_unit
                    unit = ten_thousand_unit

            if unit < ten_thousand_unit:
                unit = ten_thousand_unit * unit
        else:
            raise ValueError(f"{cn_num} 不在转化范围内")

    return output


if __name__ == '__main__':
    cn_data1 = ["一百二十三", 123]
    cn_data2 = ["一千二百三十四万五千六百七十八", 12345678]
    cn_data3 = ["一亿二千三百四十五万六千七百八十一", 123456781]
    cn_data4 = ["一千二百三十四万五千六百七十八亿一千二百三十四万五千六百七十八", 1234567812345678]

    # 正向遍历 1，用数字乘单位，然后直接把他们累加起来
    print("\n# forward cn2an 1")
    print(cn_data1[0], forward_cn2an_one(cn_data1[0]), forward_cn2an_one(cn_data1[0]) == cn_data1[1])
    print(cn_data2[0], forward_cn2an_one(cn_data2[0]), forward_cn2an_one(cn_data2[0]) == cn_data2[1])
    # # forward cn2an 1
    # 一百二十三 123 True
    # 一千二百三十四万五千六百七十八 46908 False

    # 正向遍历 2，用数字乘单位，碰到万的时候先把前面的累加起来乘以万，再直接把后面的累加起来
    print("\n# forward cn2an 2")
    print(cn_data1[0], forward_cn2an_two(cn_data1[0]), forward_cn2an_two(cn_data1[0]) == cn_data1[1])
    print(cn_data2[0], forward_cn2an_two(cn_data2[0]), forward_cn2an_two(cn_data2[0]) == cn_data2[1])
    print(cn_data3[0], forward_cn2an_two(cn_data3[0]), forward_cn2an_two(cn_data3[0]) == cn_data3[1])
    # # forward cn2an 2
    # 一百二十三 123 True
    # 一千二百三十四万五千六百七十八 12345678 True
    # 一亿二千三百四十五万六千七百八十一 1000023456781 False

    # 正向遍历 3，用数字乘单位，碰到亿的时候先把前面的累加起来乘以亿，碰到万的时候先把前面的累加起来乘以万，再直接把后面的累加起来
    # 最后把亿位加上后面的累加和
    print("\n# forward cn2an 3")
    print(cn_data1[0], forward_cn2an_three(cn_data1[0]), forward_cn2an_three(cn_data1[0]) == cn_data1[1])
    print(cn_data2[0], forward_cn2an_three(cn_data2[0]), forward_cn2an_three(cn_data2[0]) == cn_data2[1])
    print(cn_data3[0], forward_cn2an_three(cn_data3[0]), forward_cn2an_three(cn_data3[0]) == cn_data3[1])
    print(cn_data4[0], forward_cn2an_three(cn_data4[0]), forward_cn2an_three(cn_data4[0]) == cn_data4[1])
    # # forward cn2an 3
    # 一百二十三 123 True
    # 一千二百三十四万五千六百七十八 12345678 True
    # 一亿二千三百四十五万六千七百八十一 123456781 True
    # 一千二百三十四万五千六百七十八亿一千二百三十四万五千六百七十八 1234567812345678 True

    # 反向遍历 1，用数字乘单位，然后直接把他们累加起来
    print("\n# backward cn2an 1")
    print(cn_data1[0], backward_cn2an_one(cn_data1[0]), backward_cn2an_one(cn_data1[0]) == cn_data1[1])
    print(cn_data2[0], backward_cn2an_one(cn_data2[0]), backward_cn2an_one(cn_data2[0]) == cn_data2[1])
    # backward cn2an 1
    # 一百二十三 123 True
    # 一千二百三十四万五千六百七十八 46908 False

    # 反向遍历 2，用数字乘单位，然后直接把他们累加起来，碰到万或亿的时候把单位乘万或亿
    print("\n# backward cn2an 2")
    print(cn_data1[0], backward_cn2an_two(cn_data1[0]), backward_cn2an_two(cn_data1[0]) == cn_data1[1])
    print(cn_data2[0], backward_cn2an_two(cn_data2[0]), backward_cn2an_two(cn_data2[0]) == cn_data2[1])
    print(cn_data3[0], backward_cn2an_two(cn_data3[0]), backward_cn2an_two(cn_data3[0]) == cn_data3[1])
    print(cn_data4[0], backward_cn2an_two(cn_data4[0]), backward_cn2an_two(cn_data4[0]) == cn_data4[1])
    # # backward cn2an 2
    # 一百二十三 123 True
    # 一千二百三十四万五千六百七十八 12345678 True
    # 一亿二千三百四十五万六千七百八十一 123456781 True
    # 一千二百三十四万五千六百七十八亿一千二百三十四万五千六百七十八 567824685678 False

    # 反向遍历 3，用数字乘单位，然后直接把他们累加起来，碰到万或亿的时候把单位乘万或亿，碰到万亿时把单位乘万亿
    print("\n# backward cn2an 3")
    print(cn_data1[0], backward_cn2an_three(cn_data1[0]), backward_cn2an_three(cn_data1[0]) == cn_data1[1])
    print(cn_data2[0], backward_cn2an_three(cn_data2[0]), backward_cn2an_three(cn_data2[0]) == cn_data2[1])
    print(cn_data3[0], backward_cn2an_three(cn_data3[0]), backward_cn2an_three(cn_data3[0]) == cn_data3[1])
    print(cn_data4[0], backward_cn2an_three(cn_data4[0]), backward_cn2an_three(cn_data4[0]) == cn_data4[1])
    # # backward cn2an 3
    # 一百二十三 123 True
    # 一千二百三十四万五千六百七十八 12345678 True
    # 一亿二千三百四十五万六千七百八十一 123456781 True
    # 一千二百三十四万五千六百七十八亿一千二百三十四万五千六百七十八 1234567812345678 True



import unittest

from .an2cn import An2Cn


class An2CnTest(unittest.TestCase):
    def setUp(self) -> None:
        self.input_data = {
            0: ["零", "零", "零元整", "零"],
            1: ["一", "壹", "壹元整", "一"],
            11: ["十一", "壹拾壹", "壹拾壹元整", "一一"],
            1000000: ["一百万", "壹佰万", "壹佰万元整", "一零零零零零零"],
            1000054: ["一百万零五十四", "壹佰万零伍拾肆", "壹佰万零伍拾肆元整", "一零零零零五四"],
            31000054: ["三千一百万零五十四", "叁仟壹佰万零伍拾肆", "叁仟壹佰万零伍拾肆元整", "三一零零零零五四"],
            9876543298765432: [
                "九千八百七十六万五千四百三十二亿九千八百七十六万五千四百三十二",
                "玖仟捌佰柒拾陆万伍仟肆佰叁拾贰亿玖仟捌佰柒拾陆万伍仟肆佰叁拾贰",
                "玖仟捌佰柒拾陆万伍仟肆佰叁拾贰亿玖仟捌佰柒拾陆万伍仟肆佰叁拾贰元整",
                "九八七六五四三二九八七六五四三二"
            ],
            10000000000000: ["十万亿", "壹拾万亿", "壹拾万亿元整", "一零零零零零零零零零零零零零"],
            -0: ["零", "零", "零元整", "零"],
            -1: ["负一", "负壹", "负壹元整", "负一"],
            -11: ["负十一", "负壹拾壹", "负壹拾壹元整", "负一一"],
            0.000500050005005: [
                "零点零零零五零零零五零零零五零零五",
                "零点零零零伍零零零伍零零零伍零零伍",
                "零元整",
                "零点零零零五零零零五零零零五零零五"
            ],
            0.00005: ["零点零零零零五", "零点零零零零伍", "零元整", "零点零零零零五"],
            0.4321: ["零点四三二一", "零点肆叁贰壹", "肆角叁分", "零点四三二一"],
            1000054.4321: [
                "一百万零五十四点四三二一",
                "壹佰万零伍拾肆点肆叁贰壹",
                "壹佰万零伍拾肆元肆角叁分",
                "一零零零零五四点四三二一"
            ],
            1.01: ["一点零一", "壹点零壹", "壹元零壹分", "一点零一"],
            1.2: ["一点二", "壹点贰", "壹元贰角", "一点二"],
            0.01: ["零点零一", "零点零壹", "壹分", "零点零一"],
            -0.1: ["负零点一", "负零点壹", "负壹角", "负零点一"],
            -0: ["零", "零", "零元整", "零"],
            1.10: ["一点一", "壹点壹", "壹元壹角", "一点一"],
            12.0: ["十二点零", "壹拾贰点零", "壹拾贰元整", "一二点零"],
            2.0: ["二点零", "贰点零", "贰元整", "二点零"],
            0.10: ["零点一", "零点壹", "壹角", "零点一"]
        }

        self.error_input_data = [
            "123.1.1",
            "0.1零"
        ]

        self.ac = An2Cn()

    def test_an2cn(self) -> None:
        for item in self.input_data.keys():
            self.assertEqual(self.ac.an2cn(item), self.input_data[item][0])
            self.assertEqual(self.ac.an2cn(item, "low"), self.input_data[item][0])
            self.assertEqual(self.ac.an2cn(item, "up"), self.input_data[item][1])
            self.assertEqual(self.ac.an2cn(item, "rmb"), self.input_data[item][2])
            self.assertEqual(self.ac.an2cn(item, "direct"), self.input_data[item][3])

        with self.assertRaises(ValueError):
            for error_data in self.error_input_data:
                self.ac.an2cn(error_data)


if __name__ == '__main__':
    unittest.main()



import unittest

from .transform import Transform


class TransformTest(unittest.TestCase):
    def setUp(self) -> None:
        self.strict_data_dict = {
            "小王捡了100块钱": "小王捡了一百块钱",
            "用户增长最快的3个城市": "用户增长最快的三个城市",
            "小王的生日是2001年3月4日": "小王的生日是二零零一年三月四日",
            "小王的生日是2012年12月12日": "小王的生日是二零一二年十二月十二日",
            "今天股价上涨了8%": "今天股价上涨了百分之八",
            "第2天股价下降了-3.8%": "第二天股价下降了百分之负三点八",
            "抛出去的硬币为正面的概率是1/2": "抛出去的硬币为正面的概率是二分之一",
            "现在室内温度为39℃，很热啊！": "现在室内温度为三十九摄氏度，很热啊！",
            "创业板指9月9日早盘低开1.57%": "创业板指九月九日早盘低开百分之一点五七"
        }

        self.smart_data_dict = {
            "约2.5亿年~6500万年": "约250000000年~65000000年",
            "廿二日，日出东方": "22日，日出东方",
            "大陆": "大陆",
            "半斤": "0.5斤",
            "两个": "2个",
        }

        self.t = Transform()

    def test_transform(self) -> None:
        for strict_item in self.strict_data_dict.keys():
            self.assertEqual(self.t.transform(strict_item, "an2cn"), self.strict_data_dict[strict_item])
            self.assertEqual(self.t.transform(self.strict_data_dict[strict_item], "cn2an"), strict_item)

        for smart_item in self.smart_data_dict.keys():
            self.assertEqual(self.t.transform(smart_item, "cn2an"), self.smart_data_dict[smart_item])


if __name__ == '__main__':
    unittest.main()



import re

from .cn2an import Cn2An
from .an2cn import An2Cn
from .conf import UNIT_CN2AN


class Transform(object):
    def __init__(self) -> None:
        self.all_num = "零一二三四五六七八九"
        self.all_unit = "".join(list(UNIT_CN2AN.keys()))
        self.cn2an = Cn2An().cn2an
        self.an2cn = An2Cn().an2cn
        self.cn_pattern = f"负?([{self.all_num}{self.all_unit}]+点)?[{self.all_num}{self.all_unit}]+"
        self.smart_cn_pattern = f"-?([0-9]+.)?[0-9]+[{self.all_unit}]+"

    def transform(self, inputs: str, method: str = "cn2an") -> str:
        if method == "cn2an":
            inputs = inputs.replace("廿", "二十").replace("半", "0.5").replace("两", "2")
            # date
            inputs = re.sub(
                fr"((({self.smart_cn_pattern})|({self.cn_pattern}))年)?([{self.all_num}十]+月)?([{self.all_num}十]+日)?",
                lambda x: self.__sub_util(x.group(), "cn2an", "date"), inputs)
            # fraction
            inputs = re.sub(fr"{self.cn_pattern}分之{self.cn_pattern}",
                            lambda x: self.__sub_util(x.group(), "cn2an", "fraction"), inputs)
            # percent
            inputs = re.sub(fr"百分之{self.cn_pattern}",
                            lambda x: self.__sub_util(x.group(), "cn2an", "percent"), inputs)
            # celsius
            inputs = re.sub(fr"{self.cn_pattern}摄氏度",
                            lambda x: self.__sub_util(x.group(), "cn2an", "celsius"), inputs)
            # number
            output = re.sub(self.cn_pattern,
                            lambda x: self.__sub_util(x.group(), "cn2an", "number"), inputs)

        elif method == "an2cn":
            # date
            inputs = re.sub(r"(\d{2,4}年)?(\d{1,2}月)?(\d{1,2}日)?",
                            lambda x: self.__sub_util(x.group(), "an2cn", "date"), inputs)
            # fraction
            inputs = re.sub(r"\d+/\d+",
                            lambda x: self.__sub_util(x.group(), "an2cn", "fraction"), inputs)
            # percent
            inputs = re.sub(r"-?(\d+\.)?\d+%",
                            lambda x: self.__sub_util(x.group(), "an2cn", "percent"), inputs)
            # celsius
            inputs = re.sub(r"\d+℃",
                            lambda x: self.__sub_util(x.group(), "an2cn", "celsius"), inputs)
            # number
            output = re.sub(r"-?(\d+\.)?\d+",
                            lambda x: self.__sub_util(x.group(), "an2cn", "number"), inputs)
        else:
            raise ValueError(f"error method: {method}, only support 'cn2an' and 'an2cn'!")

        return output

    def __sub_util(self, inputs, method: str = "cn2an", sub_mode: str = "number") -> str:
        try:
            if inputs:
                if method == "cn2an":
                    if sub_mode == "date":
                        return re.sub(fr"(({self.smart_cn_pattern})|({self.cn_pattern}))",
                                      lambda x: str(self.cn2an(x.group(), "smart")), inputs)
                    elif sub_mode == "fraction":
                        if inputs[0] != "百":
                            frac_result = re.sub(self.cn_pattern,
                                                 lambda x: str(self.cn2an(x.group(), "smart")), inputs)
                            numerator, denominator = frac_result.split("分之")
                            return f"{denominator}/{numerator}"
                        else:
                            return inputs
                    elif sub_mode == "percent":
                        return re.sub(f"(?<=百分之){self.cn_pattern}",
                                      lambda x: str(self.cn2an(x.group(), "smart")), inputs).replace("百分之", "") + "%"
                    elif sub_mode == "celsius":
                        return re.sub(f"{self.cn_pattern}(?=摄氏度)",
                                      lambda x: str(self.cn2an(x.group(), "smart")), inputs).replace("摄氏度", "℃")
                    elif sub_mode == "number":
                        return str(self.cn2an(inputs, "smart"))
                    else:
                        raise Exception(f"error sub_mode: {sub_mode} !")
                else:
                    if sub_mode == "date":
                        inputs = re.sub(r"\d+(?=年)",
                                        lambda x: self.an2cn(x.group(), "direct"), inputs)
                        return re.sub(r"\d+",
                                      lambda x: self.an2cn(x.group(), "low"), inputs)
                    elif sub_mode == "fraction":
                        frac_result = re.sub(r"\d+", lambda x: self.an2cn(x.group(), "low"), inputs)
                        numerator, denominator = frac_result.split("/")
                        return f"{denominator}分之{numerator}"
                    elif sub_mode == "celsius":
                        return self.an2cn(inputs[:-1], "low") + "摄氏度"
                    elif sub_mode == "percent":
                        return "百分之" + self.an2cn(inputs[:-1], "low")
                    elif sub_mode == "number":
                        return self.an2cn(inputs, "low")
                    else:
                        raise Exception(f"error sub_mode: {sub_mode} !")
        except Exception as e:
            print(f"WARN: {e}")
            return inputs



import torbjorn as tbn

from .an2cn import An2Cn
from .cn2an import Cn2An

ac = An2Cn()
ca = Cn2An()

an = 9876543298765432
cn = "九千八百七十六万五千四百三十二亿九千八百七十六万五千四百三十二"


@tbn.run_time
def run_cn2an_ten_thousand_times() -> None:
    for _ in range(10000):
        result = ca.cn2an(cn)
        assert result == an


@tbn.run_time
def run_an2cn_ten_thousand_times() -> None:
    for _ in range(10000):
        result = ac.an2cn(an)
        assert result == cn


if __name__ == '__main__':
    run_cn2an_ten_thousand_times()
    run_an2cn_ten_thousand_times()



import os
import sys
from setuptools import setup, find_packages

NAME = "cn2an"
AUTHOR = "Ailln"
EMAIL = "kinggreenhall@gmail.com"
URL = "https://github.com/Ailln/cn2an"
LICENSE = "MIT License"
DESCRIPTION = "Convert Chinese numerals and Arabic numerals."

if sys.version_info < (3, 6, 0):
    raise RuntimeError(f"{NAME} requires Python >=3.6.0, but yours is {sys.version}!")

try:
    lib_py = os.path.join(NAME, "__init__.py")
    with open(lib_py, "r", encoding="utf8") as f_v:
        v_line = ""
        for line in f_v.readlines():
            if line.startswith("__version__"):
                v_line = line.strip()
                break
        exec(v_line)  # get __version__ from __init__.py
except FileNotFoundError:
    __version__ = "0.0.0"

if __name__ == "__main__":
    setup(
        name=NAME,
        version=__version__,
        author=AUTHOR,
        author_email=EMAIL,
        url=URL,
        license=LICENSE,
        description=DESCRIPTION,
        packages=find_packages(),
        include_package_data=True,
        install_requires=open("./requirements.txt", "r", encoding="utf-8").read().splitlines(),
        long_description=open("./README.md", "r", encoding="utf-8").read(),
        long_description_content_type='text/markdown',
        zip_safe=True,
        classifiers=[
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            f"License :: OSI Approved :: {LICENSE}",
            "Operating System :: OS Independent",
        ],
        python_requires=">=3.6"
    )
